#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: BufferAllocator.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/IOTools/IMonitorBufferAllocation.h"
#include "JetByteTools/IOTools/BufferAllocator.h"

#if JETBYTE_SERVER_COMMON_TRACK_BUFFER_ALLOCATION_USAGE_STATS == 1

#include "JetByteTools/Win32Tools/PerThreadUsageTracker.h"

#endif

///////////////////////////////////////////////////////////////////////////////
// CBufferAllocator
///////////////////////////////////////////////////////////////////////////////

class CBufferAllocator:
   public JetByteTools::IO::CBufferAllocator,
   private JetByteTools::IO::IMonitorBufferAllocation
{
   public :

      CBufferAllocator(
         BufferSize bufferSize,
         BufferCount maxFreeBuffers,
         bool log = true);

      CBufferAllocator(
         BufferSize bufferSize,
         BufferCount maxFreeBuffers,
         AllocationFlags flags,
         bool log = true);

      CBufferAllocator(
         const CBufferAllocator &rhs) = delete;

      ~CBufferAllocator();

      CBufferAllocator &operator=(
         const CBufferAllocator &rhs) = delete;

   private :

      /// Implement IMonitorBufferAllocation

      void OnBufferCreated() override;

      void OnBufferAllocated(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnBufferAllocationContention() override;

      void OnBufferReleaseContention() override;

      void OnBufferReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnBufferDestroyed() override;

      const bool m_log;

#if JETBYTE_SERVER_COMMON_TRACK_BUFFER_ALLOCATION_USAGE_STATS == 1

      JetByteTools::Win32::CPerThreadUsageTracker m_perThreadAllocationStats;

      JetByteTools::Win32::CPerThreadUsageTracker m_perThreadReleaseStats;

      JetByteTools::Win32::CPerThreadUsageTracker m_perThreadActiveStats;

#endif
};

///////////////////////////////////////////////////////////////////////////////
// End of file: BufferAllocator.h
///////////////////////////////////////////////////////////////////////////////
