///////////////////////////////////////////////////////////////////////////////
// File: DatagramSocketConnectionManagerCallback.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "DatagramSocketConnectionManagerCallback.h"

#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"
#include "JetByteTools/CoreTools/ToString.h"

#include "JetByteTools/CoreTools/ErrorCodeToErrorMessage.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::ToString;
using JetByteTools::Core::_tstring;
using JetByteTools::Core::Output;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::ErrorCodeToErrorMessage;
using JetByteTools::Core::IIndexedOpaqueUserData;

using JetByteTools::IO::IBuffer;
using JetByteTools::IO::CSmartBuffer;

using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::IDatagramSocket;
using JetByteTools::Socket::ConnectionClosureReason;
using JetByteTools::Socket::GetClosureReasonAsString;

///////////////////////////////////////////////////////////////////////////////
// CDatagramSocketConnectionManagerCallback
///////////////////////////////////////////////////////////////////////////////

void CDatagramSocketConnectionManagerCallback::OnSocketCreated(
   SOCKET /*s*/)
{

}

void CDatagramSocketConnectionManagerCallback::OnBindCompleted(
   IDatagramSocket & /*socket*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnPreOutgoingConnect(
   IIndexedOpaqueUserData & /*userData*/,
   const IAddress & /*address*/,
   const void * /*pUserData*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnConnectionEstablished(
   IDatagramSocket & /*socket*/,
   const IAddress & /*address*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnOutgoingConnectionFailed(
   IDatagramSocket & /*socket*/,
   const IAddress & /*address*/,
   DWORD /*lastError*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnSocketReleased(
   IIndexedOpaqueUserData & /*userData*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnReadCompleted(
   IDatagramSocket & /*socket*/,
   CSmartBuffer & /*buffer*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnReadCompletionError(
   IDatagramSocket & /*socket*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketConnectionManagerCallback::OnReadCompletionError"), lastError);
}

void CDatagramSocketConnectionManagerCallback::OnWriteCompleted(
   IDatagramSocket & /*socket*/,
   CSmartBuffer & /*buffer*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnWriteCompletionError(
   IDatagramSocket & /*socket*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketConnectionManagerCallback::OnWriteCompletionError"), lastError);
}

void CDatagramSocketConnectionManagerCallback::OnRecvFromCompleted(
   IDatagramSocket & /*socket*/,
   const IAddress & /*address*/,
   CSmartBuffer & /*buffer*/)
{
}

void CDatagramSocketConnectionManagerCallback::OnRecvFromCompletionError(
   IDatagramSocket & /*socket*/,
   const IAddress & /*address*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketConnectionManagerCallback::OnRecvFromCompletionError"), lastError);
}

void CDatagramSocketConnectionManagerCallback::OnSendToCompleted(
   IDatagramSocket & /*socket*/,
   const IAddress & /*address*/,
   CSmartBuffer & /*buffer*/)
{
   Output(_T("OnSendToCompleted"));
}

void CDatagramSocketConnectionManagerCallback::OnSendToCompletionError(
   IDatagramSocket & /*socket*/,
   const IAddress & /*address*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketConnectionManagerCallback::OnSendToCompletionError"), lastError);
}

void CDatagramSocketConnectionManagerCallback::OnMaximumConnections()
{
   Output(_T("OnMaximumConnections"));
}

void CDatagramSocketConnectionManagerCallback::OnAllConnectionsAborted(
   const DWORD numberOfConnectionsAborted)
{
   Output(_T("OnAllConnectionsAborted: ") + ToString(numberOfConnectionsAborted));
}

void CDatagramSocketConnectionManagerCallback::OnShutdownInitiated()
{
   Output(_T("OnShutdownInitiated"));
}

void CDatagramSocketConnectionManagerCallback::OnShutdownComplete()
{
   Output(_T("OnShutdownComplete"));
}

void CDatagramSocketConnectionManagerCallback::OnConnectionReset(
    IDatagramSocket & /*socket*/,
    const DWORD lastError)
{
   Output(_T("OnConnectionReset: ") + ErrorCodeToErrorMessage(lastError));
}

void CDatagramSocketConnectionManagerCallback::OnConnectionClosure(
   IDatagramSocket & /*socket*/,
   const ConnectionClosureReason reason)
{
   Output(_T("OnConnectionClosure: ") + GetClosureReasonAsString(reason));
}

void CDatagramSocketConnectionManagerCallback::OnConnectionClosed(
   IDatagramSocket & /*socket*/)
{
   Output(_T("OnConnectionClosed"));
}

void CDatagramSocketConnectionManagerCallback::OnError(
   const _tstring &message)
{
   OutputEx(_T("CDatagramSocketConnectionManagerCallback::OnError: ") + message);
}

void CDatagramSocketConnectionManagerCallback::OnError(
   const _tstring &message,
   const DWORD lastError)
{
   OutputEx(_T("CDatagramSocketConnectionManagerCallback::OnError: ") + message + _T(": ") + ErrorCodeToErrorMessage(lastError));
}

///////////////////////////////////////////////////////////////////////////////
// End of file: DatagramSocketServerCallback.cpp
///////////////////////////////////////////////////////////////////////////////
