///////////////////////////////////////////////////////////////////////////////
// File: DatagramSocketServerCallback.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "DatagramSocketServerCallback.h"

#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/ToString.h"

#include "JetByteTools/CoreTools/ErrorCodeToErrorMessage.h"

#include "JetByteTools/SocketTools/Socket.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;
using JetByteTools::Core::Output;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::ToString;
using JetByteTools::Core::IIndexedOpaqueUserData;
using JetByteTools::Core::ErrorCodeToErrorMessage;

using JetByteTools::IO::IBuffer;
using JetByteTools::IO::CSmartBuffer;

using JetByteTools::Socket::IDatagramServerSocket;
using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::ConnectionClosureReason;
using JetByteTools::Socket::GetClosureReasonAsString;

///////////////////////////////////////////////////////////////////////////////
// CDatagramSocketServerCallback
///////////////////////////////////////////////////////////////////////////////

void CDatagramSocketServerCallback::OnStartAcceptingConnections()
{
   Output(_T("OnStartAcceptingConnections"));
}

void CDatagramSocketServerCallback::OnStopAcceptingConnections()
{
   Output(_T("OnStopAcceptingConnections"));
}

void CDatagramSocketServerCallback::OnSocketCreated(
   SOCKET /*s*/)
{
}

void CDatagramSocketServerCallback::OnSocketReleased(
   IIndexedOpaqueUserData & /*userData*/)
{
}

void CDatagramSocketServerCallback::OnConnectionReset(
    IDatagramServerSocket & /*socket*/,
    DWORD lastError)
{
   Output(_T("OnConnectionReset: ") + ErrorCodeToErrorMessage(lastError));
}

void CDatagramSocketServerCallback::OnConnectionClosure(
   IDatagramServerSocket & /*socket*/,
   const ConnectionClosureReason /*reason*/)
{
   //Output(_T("OnConnectionClosure: ") + GetClosureReasonAsString(reason));
}

void CDatagramSocketServerCallback::OnConnectionClosed(
   IDatagramServerSocket & /*socket*/)
{
   //Output(_T("OnConnectionClosed"));
}

void CDatagramSocketServerCallback::OnReadCompleted(
   IDatagramServerSocket & /*socket*/,
   CSmartBuffer & /*buffer*/)
{
}

void CDatagramSocketServerCallback::OnReadCompletionError(
   IDatagramServerSocket & /*socket*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketServerCallback::OnReadCompletionError "), lastError);
}

void CDatagramSocketServerCallback::OnWriteCompleted(
   IDatagramServerSocket & /*socket*/,
   CSmartBuffer & /*buffer*/)
{
}

void CDatagramSocketServerCallback::OnWriteCompletionError(
   IDatagramServerSocket & /*socket*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketServerCallback::OnWriteCompletionError"), lastError);
}

void CDatagramSocketServerCallback::OnSendToCompleted(
   IDatagramServerSocket & /*socket*/,
   const IAddress & /*address*/,
   CSmartBuffer & /*buffer*/)
{
   Output(_T("OnSendToCompleted"));
}

void CDatagramSocketServerCallback::OnSendToCompletionError(
   IDatagramServerSocket & /*socket*/,
   const IAddress & /*address*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CDatagramSocketServerCallback::OnSendToCompletionError"), lastError);
}

void CDatagramSocketServerCallback::OnMaximumConnections()
{
   Output(_T("OnMaximumConnections"));
}

void CDatagramSocketServerCallback::OnAllConnectionsAborted(
   const DWORD numberOfConnectionsAborted)
{
   Output(_T("OnAllConnectionsAborted: ") + ToString(numberOfConnectionsAborted));
}

void CDatagramSocketServerCallback::OnShutdownInitiated()
{
   Output(_T("OnShutdownInitiated"));
}

void CDatagramSocketServerCallback::OnShutdownComplete()
{
   Output(_T("OnShutdownComplete"));
}

void CDatagramSocketServerCallback::OnIncrementPendingReceives(
   const DWORD /*pendingReceives*/)
{
}

void CDatagramSocketServerCallback::OnDecrementPendingReceives(
   const DWORD /*pendingReceives*/)
{
}

void CDatagramSocketServerCallback::OnError(
   const _tstring & message)
{
   OutputEx(_T("CDatagramSocketServerCallback::OnError: ") + message);
}

void CDatagramSocketServerCallback::OnError(
   const _tstring &message,
   const DWORD lastError)
{
   OutputEx(_T("CDatagramSocketServerCallback::OnError: ") + message + _T(": ") + ErrorCodeToErrorMessage(lastError));
}

///////////////////////////////////////////////////////////////////////////////
// End of file: DatagramSocketServerCallback.cpp
///////////////////////////////////////////////////////////////////////////////
