#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2009 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"
#include "JetByteTools/CoreTools/IQueueTimers.h"

#include <string>

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback,
   private JetByteTools::Core::IQueueTimers::Timer
{
   public :

      CSocketServer(
         const std::string &welcomeMessage,
         JetByteTools::Core::IQueueTimers &timerQueue,
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      CSocketServer(
         const CSocketServer &rhs) = delete;

      ~CSocketServer();

      CSocketServer &operator=(
         const CSocketServer &rhs) = delete;

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionClientClose(
          JetByteTools::Socket::IStreamSocket &socket) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          const DWORD lastError) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      // Implement IQueueTimers::Timer

      void OnTimer(
         UserData userData) override;

      // Our business logic

      void SetTimer(
         JetByteTools::Socket::IStreamSocket &socket);

      void EchoMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      const std::string m_welcomeMessage;

      JetByteTools::Core::IQueueTimers &m_timerQueue;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
